unit BMP;

interface

uses def, misc;

type
    TBMPFileHeader = record
      ID:          word;
      Fsize:       dword;
      Reserved1:   word;
      Reserved2:   word;
      BOfsbits:    dword;
    end;

    TBMPInfoHeader = record
      InfoSize:       dword;
      Width:          dword;
      Height:         dword;
      Planes:         word;
      BitsPerPixel:   word;
      Compression:    dword;
      ImageSize:      dword;
      XPixPerMeter:   dword;
      YPixPerMeter:   dword;
      ColorUsed:      dword;
      ColorImportant: dword;
    end;

    TRGBQuad = record
      cl_Blue:   byte;
      cl_Green:  byte;
      cl_Red:    byte;
      Reserved:  byte;
    end;

    TBMPPalette = array [0..255] of TRGBQuad;
    PBMPPalette = ^TBMPPalette;

    TBMP = record
      Width  : word;
      Height : word;
      Data   : pointer;
    end;

    PBMP = ^TBMP;

function  LoadBMPScreen (filename: string; buf: pscreen; pal: ppalette): word;
function  LoadBMP(filename: string; var bitmap: TBMP): word;
function  LoadBMPPalette(filename: string; var pal: palette): word;
procedure PutBMP(where: pointer; x,y: word; bitmap: TBMP);
procedure PutBMPCl(where: pointer; x,y: integer; bitmap: TBMP);
procedure ferror(errstr: string; var fname: file);

implementation

{}

function LoadBMPScreen (filename: string; buf: pscreen; pal: ppalette): word;
var srcfile: file;
    i,dummy: word;
    ih: TBMPInfoHeader;
    fh: TBMPFileHeader;
    quadptr: PBMPPalette;
    temp: array [0..319] of byte;
begin
  LoadBMPScreen := ERR_UNKNOWN;
  Assign(srcfile,filename);
  {$I-} Reset(srcfile,1); {$I+}
  if IOResult <> 0 then
    begin
      LoadBMPScreen := ERR_FILE_NOT_FOUND;
      exit;
    end;
  if FileSize(srcfile)>66000 then
    begin
      close(srcfile);
      LoadBMPSCreen := ERR_INV_SIZE;
      exit;
    end;
  { ---***--- }
  BlockRead (srcfile, fh, SizeOf(fh)+SizeOf(ih), dummy);
  if fh.ID<>$4D42 then
    begin
      close(srcfile);
      LoadBMPScreen := ERR_NOT_BMP;
      exit;
    end;
  if (ih.ImageSize<>64000) and (ih.ImageSize<>0) then
    begin
      close(srcfile);
      LoadBMPScreen := ERR_INV_SIZE;
      exit;
    end;
  if (ih.Width<>320) or (ih.Height<>200) then
    begin
      close(srcfile);
      LoadBMPScreen := ERR_INV_HW;
      exit;
    end;
  if ih.Compression<>0 then
    begin
      close(srcfile);
      LoadBMPScreen := ERR_BI_RLE;
      exit;
    end;
  if ih.BitsPerPixel<>8 then
    begin
      close(srcfile);
      LoadBMPScreen := ERR_NOT_256;
      exit;
    end;
  new(quadptr);
  BlockRead(srcfile, quadptr^, SizeOf(quadptr^), dummy);
  seek(srcfile, fh.BOfsbits);
  BlockRead(srcfile, buf^, SizeOf(buf^), dummy);
  if ih.ColorUsed=0 then ih.ColorUsed := 256;
  for i:=0 to ih.ColorUsed-1 do
    begin
      pal^[i,1]:=quadptr^[i].cl_Red div 4;
      pal^[i,2]:=quadptr^[i].cl_Green div 4;
      pal^[i,3]:=quadptr^[i].cl_Blue div 4;
    end;
  for i:=99 downto 0 do
    begin
      Move(buf^[i],temp,320);
      Move(buf^[199-i],buf^[i],320);
      Move(temp,buf^[199-i],320);
    end;
  dispose(quadptr);
  LoadBMPScreen:=ALL_OK;
end;


{}

function  LoadBMP(filename: string; var bitmap: TBMP): word;
var srcfile: file;
    i,dummy: word;
    padding: word;
    ih: TBMPInfoHeader;
    fh: TBMPFileHeader;
    ptrtemp: pointer;
    temp: array [0..319] of byte;
begin
  LoadBMP := ERR_UNKNOWN;
  Assign(srcfile,filename);
  {$I-} Reset(srcfile,1); {$I+}
  if IOResult <> 0 then
    begin
      LoadBMP := ERR_FILE_NOT_FOUND;
      exit;
    end;
  if FileSize(srcfile)>65078 then
    begin
      close(srcfile);
      LoadBMP := ERR_INV_SIZE;
      exit;
    end;
  { ---***--- }
  BlockRead (srcfile, fh, SizeOf(fh)+SizeOf(ih), dummy);
  if fh.ID<>$4D42 then
    begin
      close(srcfile);
      LoadBMP := ERR_NOT_BMP;
      exit;
    end;
  { zasto ovde and ?!?! kad stavim or, izbacuje, iako je velicina ok ?!?!   }
  {if (ih.ImageSize>64000) and (ih.ImageSize<>0) then
    begin
      close(srcfile);
      LoadBMP := ERR_INV_SIZE;
      exit;
    end;}
  if (ih.Width>640) or (ih.Height>400) then
    begin
      close(srcfile);
      LoadBMP := ERR_INV_HW;
      exit;
    end;
  if ih.Compression<>0 then
    begin
      close(srcfile);
      LoadBMP := ERR_BI_RLE;
      exit;
    end;
  if ih.BitsPerPixel<>8 then
    begin
      close(srcfile);
      LoadBMP := ERR_NOT_256;
      exit;
    end;
  getmem(bitmap.Data, ih.Width * ih.Height);
  bitmap.Width  := ih.Width;
  bitmap.Height := ih.Height;
  padding := 4 - ih.Width mod 4;
  if padding <> 0 then
    if MaxAvail > ih.Height * (ih.Width + padding) then
      begin
        getmem (ptrtemp, ih.Height * (ih.Width + padding));
        seek(srcfile, fh.BOfsbits);
        BlockRead(srcfile, ptrtemp^, ih.Height * (ih.Width + padding), dummy);
        for i := ih.Height div 2 - 1 downto 0 do
          begin
            Move(mem[seg(ptrtemp^):ofs(ptrtemp^)+i*(ih.Width+padding)],
                 mem[seg(bitmap.Data^):ofs(bitmap.Data^)+(ih.Height-1-i)*ih.Width],
                 ih.Width);
            Move(mem[seg(ptrtemp^):ofs(ptrtemp^)+(ih.Height-1-i)*(ih.Width+padding)],
                 mem[seg(bitmap.Data^):ofs(bitmap.Data^)+i*ih.Width],
                 ih.Width);
          end;
        if odd(ih.Height) then
          Move(mem[seg(ptrtemp^):ofs(ptrtemp^)+(ih.Height div 2)*(ih.Width+padding)],
               mem[seg(bitmap.Data^):ofs(bitmap.Data^)+(ih.Height div 2)*ih.Width],
               ih.Width);
        freemem(ptrtemp, ih.Height * (ih.Width + padding));
      end
    else
      begin
        seek(srcfile, fh.BOfsbits);
        for i := 0 to ih.Height - 1 do
          begin
            BlockRead(srcfile, mem[seg(bitmap.Data^):ofs(bitmap.Data^)+(ih.Height-i-1)*ih.Width], ih.Width, dummy);
            seek(srcfile, i*(ih.Width+padding));
          end;
    end
  else
    begin
      seek(srcfile, fh.BOfsbits);
      BlockRead(srcfile, bitmap.Data^, SizeOf(bitmap.Data^), dummy);
      for i:=99 downto 0 do
        begin
          Move(mem[seg(bitmap.Data^):ofs(bitmap.Data^)+ih.Width*i],temp,320);
          Move(mem[seg(bitmap.Data^):ofs(bitmap.Data^)+(199-i)*ih.Width],
               mem[seg(bitmap.Data^):ofs(bitmap.Data^)+i*ih.Width],320);
          Move(temp,mem[seg(bitmap.Data^):ofs(bitmap.Data^)+(199-i)*ih.Width],320);
        end;
    end;
  close(srcfile);
  LoadBMP := ALL_OK;
end;


{}

function LoadBMPPalette(filename: string; var pal: palette): word;
var srcfile: file;
    ih: TBMPInfoHeader;
    fh: TBMPFileHeader;
    dummy,i: word;
    quadptr: PBMPPalette;
begin
  LoadBMPPalette := ERR_UNKNOWN;
  Assign(srcfile,filename);
  {$I-} Reset(srcfile,1); {$I+}
  if IOResult <> 0 then
    begin
      LoadBMPPalette := ERR_FILE_NOT_FOUND;
      exit;
    end;
  BlockRead (srcfile, fh, SizeOf(fh)+SizeOf(ih), dummy);
  if fh.ID<>$4D42 then
    begin
      close(srcfile);
      LoadBMPPalette := ERR_NOT_BMP;
      exit;
    end;
  if ih.Compression<>0 then
    begin
      close(srcfile);
      LoadBMPPalette := ERR_BI_RLE;
      exit;
    end;
  if ih.BitsPerPixel<>8 then
    begin
      close(srcfile);
      LoadBMPPalette := ERR_NOT_256;
      exit;
    end;
  new(quadptr);
  BlockRead(srcfile, quadptr^, SizeOf(quadptr^), dummy);
  if ih.ColorUsed=0 then ih.ColorUsed := 256;
  for i:=0 to ih.ColorUsed-1 do
    begin
      pal[i,1]:=quadptr^[i].cl_Red div 4;
      pal[i,2]:=quadptr^[i].cl_Green div 4;
      pal[i,3]:=quadptr^[i].cl_Blue div 4;
    end;
  dispose(quadptr);
  close(srcfile);
  LoadBMPPalette := ALL_OK;
end;

{}

procedure PutBMP(where: pointer; x,y: word; bitmap: TBMP);
var Width, Height: word;
    i: word;
begin
  Width:=bitmap.Width;
  Height:=bitmap.Height;
  for i:=0 to Height-1 do
    Move(Mem[seg(bitmap.Data^):ofs(bitmap.Data^)+i*Width],Mem[seg(where^):ofs(where^)+(y+i)*320+x],Width);
end;

{}

procedure PutBMPCl(where: pointer; x,y: integer; bitmap: TBMP);
var i: word;
    crtase: boolean;
    xstart, ystart: word;
    xsrc, ysrc: word;
    xclip, yclip: word;
    x2, x3, x4,
    y2, y3, y4: integer;
begin
  crtase := false;
  x2 := x + bitmap.Width;
  x3 := x;
  x4 := x2;
  y2 := y;
  y3 := y + bitmap.Height;
  y4 := y3;
  if (x<=0) or (x2>=0) or (x<=319) or (x2>=319)
    then if (y<=0) or (y3>=0) or (y<=199) or (y3>=199)
      then crtase := TRUE;
  { da li je objekat makar jednim delom u okviru ekana }
  if not crtase then
    if (x>0) and (y>0) and (x<320) and (y<200) then crtase := TRUE;
  if not crtase then
    if (x2>0) and (y2>0) and (x2<320) and (y2<200) then crtase := TRUE;
  if not crtase then
    if (x3>0) and (y3>0) and (x3<320) and (y3<200) then crtase := TRUE;
  if not crtase then
    if (x4>0) and (y4>0) and (x4<320) and (y4<200) then crtase := TRUE;
  if crtase then
    begin
      if x < 0 then
        begin
          xstart := 0;
          xsrc := abs(x);
          if x + bitmap.Width > 320
          then
            xclip := 320
          else
            xclip := x + bitmap.Width;
        end
      else
        begin
          xstart := x;
          xsrc := 0;
          if bitmap.Width > 320 - x
          then
            xclip := 320 - x
          else
            xclip := bitmap.Width;
        end;
      if y < 0 then
        begin
          ystart := 0;
          ysrc := abs(y);
          if y + bitmap.Height > 200
          then
            yclip := 200
          else
            yclip := y + bitmap.Height;
        end
      else
        begin
          ystart := y;
          ysrc := 0;
          if bitmap.Height > 200 - y
          then
            yclip := 200 - y
          else
            yclip := bitmap.Height;
        end;

      for i := 0 to yclip-1 do
        move(mem[seg(bitmap.Data^):ofs(bitmap.Data^)+(i+ysrc)*bitmap.Width+xsrc],
             mem[seg(where^):ofs(where^)+(i+ystart)*320+xstart],
             xclip);
  end;
end;

{}

procedure ferror(errstr: string; var fname: file);
begin
     writeln('Error: '+errstr);
     close(fname);
     halt(1);
end;


{}

end.